@@ -252,20 +252,106 @@ def xyz_to_lab(c):
     return xyz_to_lab_d65(c)
 
 
-#
+# Adapted from https://github.com/lovro-i/CIEDE2000/blob/master/ciede2000.py
+# via The CIEDE2000 Color-Difference Formula: Implementation Notes, Supplementary Test Data, and Mathematical Observations
+# Gaurav Sharma, Wencheng Wu, Edul N. Dalal
+@jit(nopython=True, fastmath=True, cache=True)
+def _ciede2000(lab_c1, lab_c2):
+    L1, a1, b1 = lab_c1[0], lab_c1[1], lab_c1[2]
+    L2, a2, b2 = lab_c2[0], lab_c2[1], lab_c2[2]
+    C1 = np.sqrt(a1 ** 2 + b1 ** 2)
+    C2 = np.sqrt(a2 ** 2 + b2 ** 2)
+    C_ave = (C1 + C2) / 2
+    G = 0.5 * (1 - np.sqrt(C_ave ** 7 / (C_ave ** 7 + 6103515625)))
+    L1_, L2_ = L1, L2
+    a1_, a2_ = (1 + G) * a1, (1 + G) * a2
+    b1_, b2_ = b1, b2
+    C1_ = np.sqrt(a1_ ** 2 + b1_ ** 2)
+    C2_ = np.sqrt(a2_ ** 2 + b2_ ** 2)
+    if b1_ == 0 and a1_ == 0:
+        h1_ = 0
+    elif a1_ >= 0:
+        h1_ = np.arctan2(b1_, a1_)
+    else:
+        h1_ = np.arctan2(b1_, a1_) + 2 * np.pi
 
-# [ 47.98194319  -3.19681298 -39.3202402 ]
-if __name__ == '__main__':
-    #     # print (np.array([0.12156863, 0.46666667, 0.70588235]))
-    tmp = srgb_to_lrgb(np.array([0.12156863, 0.46666667, 0.70588235]))
-    print(tmp)
-    tmp = lrgb_to_xyz(tmp)
-    print(tmp)
-    tmp = xyz_to_lab(tmp)
-    print(tmp)
-    tmp = lab_to_xyz(tmp)
-    print(tmp)
-    tmp = xyz_to_lrgb(tmp)
-    print(tmp)
-    tmp = lrgb_to_srgb(tmp)
-    print(tmp)
+    if b2_ == 0 and a2_ == 0:
+        h2_ = 0
+    elif a2_ >= 0:
+        h2_ = np.arctan2(b2_, a2_)
+    else:
+        h2_ = np.arctan2(b2_, a2_) + 2 * np.pi
+    dL_ = L2_ - L1_
+    dC_ = C2_ - C1_
+    dh_ = h2_ - h1_
+    if C1_ * C2_ == 0:
+        dh_ = 0
+    elif dh_ > np.pi:
+        dh_ -= 2 * np.pi
+    elif dh_ < -np.pi:
+        dh_ += 2 * np.pi
+    dH_ = 2 * np.sqrt(C1_ * C2_) * np.sin(dh_ / 2)
+    L_ave = (L1_ + L2_) / 2
+    C_ave = (C1_ + C2_) / 2
+    _dh = np.abs(h1_ - h2_)
+    _sh = h1_ + h2_
+    C1C2 = C1_ * C2_
+    if _dh <= np.pi and C1C2 != 0:
+        h_ave = (h1_ + h2_) / 2
+    elif _dh > np.pi and _sh < 2 * np.pi and C1C2 != 0:
+        h_ave = (h1_ + h2_) / 2 + np.pi
+    elif _dh > np.pi and _sh >= 2 * np.pi and C1C2 != 0:
+        h_ave = (h1_ + h2_) / 2 - np.pi
+    else:
+        h_ave = h1_ + h2_
+    T = 1 - 0.17 * np.cos(h_ave - np.pi / 6) + 0.24 * np.cos(2 * h_ave) + 0.32 * np.cos(
+        3 * h_ave + np.pi / 30) - 0.2 * np.cos(4 * h_ave - 63 * np.pi / 180)
+    h_ave_deg = h_ave * 180 / np.pi
+    if h_ave_deg < 0:
+        h_ave_deg += 360
+    elif h_ave_deg > 360:
+        h_ave_deg -= 360
+    dTheta = 30 * np.exp(-(((h_ave_deg - 275) / 25) ** 2))
+    R_C = 2 * np.sqrt(C_ave ** 7 / (C_ave ** 7 + 6103515625))
+    S_C = 1 + 0.045 * C_ave
+    S_H = 1 + 0.015 * C_ave * T
+    Lm50s = (L_ave - 50) ** 2
+    S_L = 1 + 0.015 * Lm50s / np.sqrt(20 + Lm50s)
+    R_T = -np.sin(dTheta * np.pi / 90) * R_C
+    k_L, k_C, k_H = 1, 1, 1
+    f_L = dL_ / k_L / S_L
+    f_C = dC_ / k_C / S_C
+    f_H = dH_ / k_H / S_H
+    dE_00 = np.sqrt(f_L ** 2 + f_C ** 2 + f_H ** 2 + R_T * f_C * f_H)
+    return dE_00
+
+
+def my_ciede2000(c1, c2):
+    c1 = np.array(c1)
+    c2 = np.array(c2)
+    return _ciede2000(c1, c2)
+
+
+#
+#
+# # [ 47.98194319  -3.19681298 -39.3202402 ]
+# if __name__ == '__main__':
+#     #     # print (np.array([0.12156863, 0.46666667, 0.70588235]))
+#     tmp = srgb_to_lrgb(np.array([0.82156863, 0.26666667, 0.30588235]))
+#     print(tmp)
+#     tmp = lrgb_to_xyz(tmp)
+#     print(tmp)
+#     tmp = xyz_to_lab(tmp)
+#     print(tmp)
+#     tmp = lab_to_xyz(tmp)
+#     print(tmp)
+#     tmp = xyz_to_lrgb(tmp)
+#     print(tmp)
+#     tmp = lrgb_to_srgb(tmp)
+#     print(tmp)
+#     # [ 47.98083149  -3.20353037 -39.33214908]
+#     # [49.87586907 56.02126521 25.91319689]
+#     tmp = my_ciede2000([47.98083149,  -3.20353037, -39.33214908], [49.87586907, 56.02126521, 25.91319689])
+#     print(tmp)
+#     tmp = ciede2000((47.98083149,  -3.20353037, -39.33214908), (49.87586907, 56.02126521, 25.91319689))
+#
